﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;

namespace WorkflowScript_ImpermoCountLogisticUnitsScript
{
    /// <summary>
    /// Impermo
    /// </summary>
    public class WorkflowScript_ImpermoCountLogisticUnitsScript : WorkflowInstanceScriptBase
    {


        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        // Output parameters

        // Sub-flows
        // <none>



        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_ImpermoCountLogisticUnitsScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_ImpermoCountLogisticUnitsScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            string initialErrorKey = null;
            Message msg = null;

            string query = null;
            bool goBack = false;
            string selectedLocationCode = null;
            string buttonKeySelectPackingTask = null;
            DataSet dsLocations = null;
            double numberOfLogisticUnits = 0;
            string selectedLocationName = null;
            DataSet dsCheck = null;


            //add table
            query = BuildQuery.AddUpdateTable();
            sboProviderService.RunView(false, null, null, query);

            SboUserInfo userInfo = sboProviderService.GetCurrentUser();

        StepIdentifyLocation:

            #region SCAN LOCATION

            // Initialise the cache list

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IScanStringValueScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SCAN_LOCATION",
                "AllowedButtonKeys", new string[] { "MSG_BUTTON_SELECT_LOCATION" },
                "ForceDataEntry", true,
                "AllowToGoBack", true,
                "DefaultValue", "--> BarCode <--"));

            msg = WaitForMessage();
            // Reset latest error
            initialErrorKey = null;
            selectedLocationCode = null;
            selectedLocationName = null;
            buttonKeySelectPackingTask = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectPackingTask = ExtractParameter<string>(msg.Parameters, "buttonKey");

                switch (buttonKeySelectPackingTask)
                {
                    case "MSG_BUTTON_SELECT_LOCATION":
                        goto Step_SelectLocation;
                    default: break;
                }
            }
            else if (msg.Name.EndsWith(".StringScanned"))
            {
                selectedLocationCode = ExtractParameter<string>(msg.Parameters, "stringValue");

                goto Step_SelectLocation;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                return;
            }

            goto StepIdentifyLocation;

            #endregion

        Step_SelectLocation:

            #region SELECT LOCATION

            // Get needed data
            query = BuildQuery.GetAllLocation(null, selectedLocationCode);
            dsLocations = sboProviderService.RunView(false, null, null, query);

            if (GetNumberOfRows(dsLocations) == 0)
            {
                initialErrorKey = "MSG_ERROR_NOT_A_VALID_LOCATION";
                goto StepIdentifyLocation;
            }

            if (GetNumberOfRows(dsLocations) == 0 &&
                !string.IsNullOrEmpty(buttonKeySelectPackingTask))
            {
                //autoselect
                selectedLocationCode = Convert.ToString(dsLocations.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
                selectedLocationName = Convert.ToString(dsLocations.Tables[0].Rows[0][LocationDataSet.Columns.DESCRIPTION.Name]);
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                "LocationDS", dsLocations));

            msg = WaitForMessage();
            // Reset latest error
            initialErrorKey = null;
            selectedLocationCode = null;

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                selectedLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                DataRow[] rows = dsLocations.Tables[0].Select(string.Format("{0} = '{1}'", LocationDataSet.Columns.CODE.Name, selectedLocationCode));
                if (rows.Length > 0)
                {
                    selectedLocationName = Convert.ToString(rows[0][LocationDataSet.Columns.DESCRIPTION.Name]);
                }

                goto Step_EnterNumberOfLogisticUnits;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                // We go back on previous step
                goto StepIdentifyLocation;
            }

            goto Step_SelectLocation;

            #endregion

        Step_EnterNumberOfLogisticUnits:

            #region ENTER NUMBER OF LOGISTIC UNITS

            numberOfLogisticUnits = 0;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "Vul het aantal logistieke eenheden in",
                    "InitialQuantity", 0,
                    "NumberOfDigitsAfterDecimal", 0,
                    "MaximumQuantity", -1,
                    "UOM", "Paletten",
                    "AllowZero", true,
                    "DocNum", string.Format("{0} ({1})", selectedLocationName, selectedLocationCode)
                   ));
            msg = WaitForMessage();

            // reset error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                numberOfLogisticUnits = ExtractParameter<double>(msg.Parameters, "quantity");

                goto Step_CheckIfAlreadyCounted;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto StepIdentifyLocation;

            }


            #endregion

        Step_CheckIfAlreadyCounted:

            #region CHECK ALREADY COUNTED

            query = BuildQuery.GetCountedLocation(selectedLocationCode);
            dsCheck = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsCheck) > 0)
            {
                //already counted, so ask user if he wants to overwrite
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                  DefaultCultureInfo.Get(), BuildParamCollection(
                      "TitleKey", "Locatie reeds geteld",
                      "MessageKey", @"De locatie is reeds geteld. Wenst u de data te overschrijven?"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    //delete old date
                    query = BuildQuery.DeleteCountedLocation(selectedLocationCode);
                    sboProviderService.RunView(false, null, null, query);

                    goto Step_WriteData;
                }
                else
                {
                    goto StepIdentifyLocation;
                }
            }

            goto Step_WriteData;


            #endregion

        Step_WriteData:

            #region WRITE DATA

            query = BuildQuery.InsertCountedLocation(selectedLocationCode, numberOfLogisticUnits, userInfo.Username);
            sboProviderService.RunView(false, null, null, query);


            goto StepIdentifyLocation;

            #endregion



            return;

        }

        #endregion

        #region HELPERS



        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }
        #endregion


        #region QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        private class BuildQuery
        {
            /// <summary>
            /// Gets all the active location.
            /// </summary>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <param name="locationCode">The location code.</param>
            /// <returns>
            /// The sql statement to get all the active location
            /// </returns>
            public static string GetAllLocation(string warehouseCode, string locationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(warehouseCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));
                }

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.Code.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));

                return strBuilder.ToString();
            }


            /// <summary>
            /// Adds the update table.
            /// </summary>
            /// <returns></returns>
            public static string AddUpdateTable()
            {
                return @"
                if not exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[PMX_IMP_CLU]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
                CREATE TABLE [dbo].[PMX_IMP_CLU] (
	                [InternalKey] [int] IDENTITY(1,1) NOT NULL,
	                [UserName] [nvarchar] (50) NULL ,
	                [CreateDateTime] [datetime] NOT NULL,
	                [NumberOfLUID] [numeric](19, 6) NOT NULL ,
                    [StorLocCode] [nvarchar] (50) NOT NULL ,
	                CONSTRAINT [PK_PMX_IMP_CLU] PRIMARY KEY CLUSTERED 
	                (
		                [InternalKey] ASC
	                )
	                ON [PRIMARY]
                ) ON [PRIMARY]

                ";
            }



            /// <summary>
            /// Gets the counted location.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <returns></returns>
            public static string GetCountedLocation(string locationCode)
            {
                return string.Format(@"
                SELECT * FROM [PMX_IMP_CLU] WHERE StorLocCode = '{0}'", locationCode);
            }

            /// <summary>
            /// Deletes the counted location.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <returns></returns>
            public static string DeleteCountedLocation(string locationCode)
            {
                return string.Format(@"
                DELETE FROM [PMX_IMP_CLU] WHERE StorLocCode = '{0}'", locationCode);
            }


            /// <summary>
            /// Inserts the counted location.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <param name="numberOfLUID">The number of LUID.</param>
            /// <param name="userSign">The user sign.</param>
            /// <returns></returns>
            public static string InsertCountedLocation(string locationCode, double numberOfLUID, string userName)
            {
                return string.Format(@"
                INSERT INTO [PMX_IMP_CLU] (StorLocCode, NumberOfLUID, UserName , CreateDateTime ) VALUES ('{0}', {1}, '{2}', {3})",
                    locationCode, SqlCommandHelper.FormatAndEscapeColumnValue(numberOfLUID, CultureInfo.InvariantCulture), userName, SqlCommandHelper.FormatAndEscapeColumnValue(DateTime.Now, CultureInfo.InvariantCulture));
            }
        }

        #endregion
    }
}

